"""The main parser for finding and checking links

The LinkResults class expects a URL as it's only argument. It will return a 
dictionary of all links on the given html page which is stores in the object.results 
attribute. The returning dictionary consists of the link (dict value) and the http 
response (dict key).

    Typical usage example:

    results = LinkResults(url).results
"""

import requests
from bs4 import BeautifulSoup


class LinkResults:
    def __init__(self, url: str):
        if url.endswith("/"):
            self.base_url = url
        else:
            self.base_url = url + "/"

        self.all_atags = self.find_all_atags(self.base_url)
        self.results = self.build_results_dictionary()

    def __str__(self) -> str:
        return f"All links for {self.base_url}"

    def check_link_for_http_scheme(self, href: str) -> str:
        """Checks a link for http scheme.
        
        If a qualified URL, nothing is done. If a relative link, then a full URL 
        is built using the base URL.

        Args:
            href: The hypertext reference of a give anchor tag.

        Returns:
            A full qualifying URL
         """

        if href.startswith(self.base_url):
            return href
        elif href.startswith("/"):
            href = self.base_url + href.lstrip("/")
            return href
        else:  # This catches any href set to '#'
            return None  # TODO: Deal with ./ or ../ relative links.

    def find_all_atags(self, url: str):
        """Find all anchor tags on a given URL.
        
        Args:
            url: A URL string

        Returns:
            A bs4.element.ResultSet object
        """

        r = requests.get(url)
        if r.status_code == 200:
            soup = BeautifulSoup(r.content, "html.parser")
            return soup.find_all("a")

    def build_results_dictionary(self) -> dict:
        """Build the final results dictionary.
        
        Once all_tags has been populated, the final results dictionary is build by 
        testing each link. 

        Returns:
            A dictionary with the key = the HTTP response, 
            and the value = a List of URLs that achieved that response code.
        """

        results = dict()

        try:
            for tag in self.all_atags:
                href = tag.get("href")
                parsed_url = self.check_link_for_http_scheme(href)
                if parsed_url is not None:
                    parsed_url_status_code = requests.get(parsed_url).status_code

                    if parsed_url_status_code in results:
                        results[parsed_url_status_code].append(parsed_url)
                    else:
                        results[parsed_url_status_code] = [parsed_url]
        except:
            pass

        return results



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pyanchor']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4', 'requests', 'typer', 'lxml']

entry_points = \
{'console_scripts': ['pyanchor = pyanchor.cli:app']}

setup(name='pyanchor',
      version='0.3.1',
      description='Check you site for broken links!',
      author='Ricky White',
      author_email='ricky@whitelionmedia.com',
      url='https://github.com/EndlessTrax/pyanchor/',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6',
     )



import pytest
from typer.testing import CliRunner

from pyanchor.cli import app
from pyanchor.link_checker import LinkResults


runner = CliRunner()


@pytest.fixture
def example_object():
    return LinkResults("http://127.0.0.1:5000/")


def test_exception_on_invalid_url_http_scheme():
    result = runner.invoke(app, ["google.com"])
    assert result.exit_code == 1


def test_successful_result_prints():
    results = runner.invoke(app, ["http://127.0.0.1:5000/", "--verbose"])
    assert "[ 200 ] - http://127.0.0.1:5000/about" in results.output


def test_failing_result_prints():
    results = runner.invoke(app, ["http://127.0.0.1:5000/"])
    assert "[ 500 ] - http://127.0.0.1:5000/500" in results.output


def test_link_result_class_str(example_object):
    assert str(example_object) == "All links for http://127.0.0.1:5000/"


def test_results_is_dict(example_object):
    assert isinstance(example_object.results, dict)


def test_sitemap_successful_result_prints():
    results = runner.invoke(
        app, ["http://127.0.0.1:5000/sitemap.xml", "--sitemap", "--verbose"]
    )
    assert "[ 200 ] - http://127.0.0.1:5000/about" in results.output


def test_sitemap_failing_result_prints():
    results = runner.invoke(app, ["http://127.0.0.1:5000/sitemap.xml", "--sitemap"])
    assert "[ 500 ] - http://127.0.0.1:5000/500" in results.output
